<?php
namespace Teknisa\COZ\Services;

class Pla10610Service{

	protected $environment;
    protected $util;
    protected $pla10610remoteRepository;

	public function __construct($environment, $util, $pla10610remoteRepository, $crudService){
        $this->environment = $environment;
        $this->util = $util;
        $this->repository = $pla10610remoteRepository;
        $this->crudService = $crudService;
    }

    public function preencheCampos(&$row){
        $row['NRORG']      = $this->environment->getNrOrgTrab();
        $row['CDOPERADOR']      = $this->environment->getCdOperador();
        $row['CDGRUPFILI'] = $row['CDGRUPFILI'] === '' ? 'T' : $row['CDGRUPFILI']; 
        
        $row['CDCENTRAL'] = $row['NMCENTRAL'];
        
        $row['CDCENTRAL']       = $this->retornaCentrais((array)$row);
        
        if(!empty($row['CDFILIAGREGA'])) $row['LISTAFILIAGREGA'] = join('_', $row['CDFILIAGREGA']);
        else $row['LISTAFILIAGREGA'] = $this->retornaFiliaisagregadas((array)$row);
        
        if(!empty($row['CDGRUPSERV'])) $row['LISTAGRUPSERV'] = join('_', $row['CDGRUPSERV']);
        else $row['LISTAGRUPSERV']   = $this->retornaGrupserv((array)$row);
        
        if(!empty($row['FILISERV'])) $row['LISTASERVFILI'] = join('_', $row['FILISERV']);
        else $row['LISTASERVFILI']   = $this->retornaServicoporfiliais((array)$row);
    }

    private function retornaCentrais($row){
        if(empty($row['CDCENTRAL'])){
            $fieldsToFilter   = array("CDOPERADOR");
            $paramObjectArray = $this->buildFilterArrayFromRow($row, $fieldsToFilter);
            
            $rows = $this->crudService->getFilteredDataSet("filialcentral", $paramObjectArray)->getRows();
            $row['CDCENTRAL'] = array_column($rows, "CDFILIAL");
         }
        return join('_', $row['CDCENTRAL']);
    }

    private function retornaFiliaisagregadas($row){ 
        $fieldsToFilter = array("NRORG",
                                "CDGRUPFILI",
                                "CDCENTRAL");
        
        $paramObjectArray       = $this->buildFilterArrayFromRow($row, $fieldsToFilter);
        $rows = $this->crudService->getFilteredDataSet("filiaisagregadas", $paramObjectArray)->getRows();
        $row['CDFILIAGREGA'] = array_column($rows, "CDFILIAL");
        return join('_', $row['CDFILIAGREGA']);
    }

    private function retornaGrupserv($row){
    	if(empty($row['CDGRUPSERV'])){   
    	    $rows = $this->crudService->getFilteredDataSet("grupserv", array())->getRows();
            $row['CDGRUPSERV'] = array_column($rows, "CDGRUPSERV");
        }
        return join('_', $row['CDGRUPSERV']);
    }

    private function retornaServicoporfiliais($row){
    	if(empty($row['CDSERVFILI'])){
            $row['CDGRUPSERV']   = $row['LISTAGRUPSERV'];
            $row['CDFILIAGREGA'] = $row['LISTAFILIAGREGA'];

            $fieldsToFilter = array("NRORG", 
                                    "CDCENTRAL",
                                    "CDGRUPFILI",
                                    "CDGRUPSERV",
                                    "CDFILIAGREGA");

            $paramObjectArray = $this->buildFilterArrayFromRow($row, $fieldsToFilter);
            $rows = $this->crudService->getFilteredDataSet("servico_por_filiais", $paramObjectArray)->getRows();
            $row['CDSERVFILI'] = array_column($rows, "FILISERV");;
        }
        return join('_', $row['CDSERVFILI']);
    }

	private function concatArray($array, $key = false){
        if ($key){
            $string = '';
            foreach ($array as $item) {
              $string .= $item[$key].'_';
            }
            return trim($string, '_');
        }
        else return implode("_", $array);
    }

    /*
     * Constroi um array de arrays a partir de uma row, no formato chave=>valor,
     * e um array de strings especificando quais das colunas deverão ser usadas
     * pega todos os valores de $row com chaves presentes em $filterKeys
     *  o array_flip faz com que os valores de $filterKeys sejam chaves
     */
    public function buildFilterArrayFromRow($row, $filterKeys){
        $paramValueArray = array_intersect_key($row, array_flip($filterKeys));

        return array_map(function($key, $value){
            return array(
                "columnName" => $key, 
                "operator" => "=",
                "value" => $value); 
                
        }, array_keys($paramValueArray), $paramValueArray);
    }
}